"""
agmem constants - central definition of domain literals.
"""

# Memory type subdirectories in current/
MEMORY_TYPES = (
    "episodic",
    "semantic",
    "procedural",
    "checkpoints",
    "session-summaries",
)
